###############################################################################
#   File      : SimManager.tcl
#   Version   : 1.1
#   Date      : January 25, 2013
#   Revisions : -
#   Author    : Roozbeh Dolatshahi (r.dolatshahi@puz.de)
#   Copyright : P+Z Engineering, GmbH. All rights reserved.
###############################################################################


set auto_path;
lappend auto_path "."; 				# Extend the auto_path so that it contains the HWT package
package require hwt; 				# Append the HWT package for using the TK widgets
package require http;				# Import HTTP package
package require hwat;				# Import HWAT package
package require dom;				# Import DOM package for XML processing
package require Tktable;			# import Tktable for drawing tables

set WEB_APP "servlet/rest";			# Set web application context as a global variable
set COOKIES "";					# Set the cookies available as a global variable to all functions
global PROJECTS
global MODELS
global SELECTED_MODEL
global SELECTED_PROJECT


# Procedure to load array values between sessions
proc persistentArray {arrName {filename {}}} {
    upvar 1 $arrName arr
    array set arr {} ;# to make sure it exists, and is an array
    if {$filename == ""} {set filename $arrName.txt}
    set filename [file join [pwd] $filename]
    if [file exists $filename] {
        set fp [open $filename]
        array set arr [read $fp]
        close $fp
    }
}

# Procedure to save array values to file
proc persistSave {filename arrName} {
    upvar 1 $arrName arr
    set filename [file join [pwd] $filename]
    if [file exists $filename] {
        file delete $filename
    }
    set    fp [open $filename w]
    foreach key [array names arr] {
        puts  $fp [list $key $arr($key)]
    }
    close $fp
}

# get the selected project or model from corresponding tables
proc tblCmd { mytype row } {

	puts "Handler.......Call me"

    global PROJECTS
    global MODELS
    global SELECTED_PROJECT
    global SELECTED_MODEL

    if {[string equal $mytype project]} {
        foreach key [array name PROJECTS] {
            if [string equal [lindex [split $PROJECTS($key) ,] 0] [lindex [lindex $row 0] 0]] {
                set SELECTED_PROJECT $key
            }
        }
    } else {
        foreach key [array name MODELS] {
            if [string equal [lindex $MODELS($key) 0] [lindex [lindex $row 0] 0] ] {
                set SELECTED_MODEL $key
            }
        }
    } 
}

proc LoadModel {args} {
    global WEB_APP
    global COOKIES
    global MODELS
    global SELECTED_MODEL

    # Local proxy configuration
    #http::config -proxyhost 192.0.2.150 -proxyport 3128
    # Get object ID for the chosen project name
    #set modelId [lindex [array get MODELS $::modelName] end]

    # get the full url for vault query
    set vaultUrl "$::url/$WEB_APP/vault?oid=$SELECTED_MODEL&expr=files.model"

    set token4 [http::geturl $vaultUrl -timeout 30000 -binary true -strict false -headers [list Cookie [join $COOKIES {;}]]]
    set status4 [http::status $token4]
    set answer4 [http::data $token4]
    set filename "./[lindex $MODELS($SELECTED_MODEL) 0].hm"
    set modelFileId [open $filename "w"]
    fconfigure $modelFileId -translation binary
    puts $modelFileId $answer4
    http::cleanup $token4
    close $modelFileId
    *readfile $filename
    *answer yes
    destroy $args;
}

proc chooseModel {args} {
    global MODELS

    # Define the top level widow
    hwt::CreateWindow mythirdlevel \
     -windowtitle "Choose Model" \
     -noGeometrySaving \
     -cancelButton Cancel \
     -acceptButton OK \
     -acceptFunc LoadModel \
     -defaultButton Accept \
     -fixedSize 200 150 \
     -post;

     # Define the recessed window area
     set recess [hwt::WindowRecess mythirdlevel];

    # Insert section Label
    set section1 [label $recess.section1 \
     -text "Found Models:" \
     -justify left \
     -anchor nw \
     -font [hwt::AppFont]];

    # Pack the label
    pack $section1 -side top -anchor nw;

    # Add padding before the next control
    hwt::AddPadding $recess -side top -height [hwt::DluHeight 4];

    # Define the frame to hold the widgets
    set frame1 [frame $recess.frame1];

    # pack the frame
    pack $frame1 -side top -anchor nw;

    # Add indent spacing to the frame
    hwt::AddPadding $frame1 -side left -width [hwt::DluWidth 4];

    global tableVals
    array unset tableVals
    array set tableVals {}
    set nRow 0
    foreach key [array names MODELS] {
         set tableVals($nRow,0) [lindex $MODELS($key) 0]
         set tableVals($nRow,1) [lindex $MODELS($key) 1]
         set nRow [expr $nRow + 1]
    }

table $frame1.modTable -rows $nRow -cols 2 \
                        -selectmode single \
                        -selecttype row \
                        -colstretchmode all \
                        -rowstretchmode all \
                        -state disabled \
                        -variable tableVals \
                        -justify center \
                        -borderwidth 2 \
                        -resizeborders none \
                        -width 8 -height 8 \
                        -colwidth 30 \
                        -exportselection true -cache 1 \
                        -selectioncommand [list tblCmd model %s]

pack $frame1.modTable

}

proc GoToModelWindow {args} {
    global WEB_APP
    global COOKIES
    global PROJECTS
    global MODELS
    global SELECTED_PROJECT

    # Local proxy configuration
    # http::config -proxyhost 192.0.2.150 -proxyport 3128
    # Get object ID for the chosen project name
    set SELECTED_PROJECT [lindex [split $SELECTED_PROJECT :] end-1]

    # get the full url for Model query
    set modelUrl "$::url/$WEB_APP/query?type=MeshModel&expr=\[project.objectId==\'$SELECTED_PROJECT\'\]&f=label&f=description"

    set token3 [http::geturl $modelUrl -timeout 30000 -strict false -headers [list Cookie [join $COOKIES {;}]]]
    set status3 [http::status $token3]
    set answer3 [http::data $token3]
    #puts $answer3
    http::cleanup $token3

    array unset MODELS
    array set MODELS {}

    # Parse the returned XML file
    set modelXml [::dom::tcl::parse $answer3]
    # Get the object list node
    set objNode [::dom::tcl::selectNode $modelXml "objList"]
    # Get the object list children nodes
    set objListChildren [::dom::node children $objNode]
    foreach child $objListChildren {
        set tempName [::dom::node cget $child -nodeName]
        if {[string equal -nocase $tempName obj]} {
            set tempAttr [::dom::node cget $child -attributes]
            set tempObjId [lindex [array get $tempAttr oid] end]
            set objListChildren2 [::dom::node children $child]
            set tempSpec ""
            foreach grandchild $objListChildren2 {
                set tempName2 [::dom::node cget $grandchild -nodeName]
                if {[string equal -nocase $tempName2 attr]} {
                    lappend tempSpec [::dom::node stringValue $grandchild]
                }
            }
            set MODELS($tempObjId) $tempSpec
        }
    }
    destroy $args;
    chooseModel
}

proc chooseProject {args} {
	
    global PROJECTS

    # Define the top level widow
    hwt::CreateWindow mysecondlevel \
     -windowtitle "Choose Project" \
     -noGeometrySaving \
     -cancelButton Cancel \
     -acceptButton OK \
     -acceptFunc GoToModelWindow \
     -defaultButton Accept \
     -fixedSize 400 200 \
     -post;

    # Define the recessed window area
    set recess [hwt::WindowRecess mysecondlevel];

    # Insert section Label
    set section1 [label $recess.section1 \
     -text "Found Projects:" \
     -justify left \
     -anchor nw \
     -font [hwt::AppFont]];

    # Pack the label
    pack $section1 -side top -anchor nw;

    # Add padding before the next control
    hwt::AddPadding $recess -side top -height [hwt::DluHeight 4];


    # Define the frame to hold the widgets
    set frame1 [frame $recess.frame1];

    # pack the frame
    pack $frame1 -side top -anchor nw;

    # Add indent spacing to the frame
    hwt::AddPadding $frame1 -side left -width [hwt::DluWidth 4];

    global tableVals
    array unset tableVals
    array set tableVals {}
    set nRow 0
    foreach key [array names PROJECTS] {
         set tableVals($nRow,0) [lindex [split $PROJECTS($key) ,] 0]
         set tableVals($nRow,1) [string trimright "[lindex [split $PROJECTS($key) ,] 1] [lindex [split $PROJECTS($key) ,] 2]"]
         set nRow [expr $nRow + 1]
    }


table $frame1.projTable -rows $nRow -cols 2 \
                        -selectmode single \
                        -selecttype row \
                        -colstretchmode all \
                        -rowstretchmode all \
                        -state disabled \
                        -variable tableVals \
                        -justify center \
                        -borderwidth 2 \
                        -resizeborders none \
                        -width 8 -height 8 \
                        -colwidth 30 \
                        -exportselection true -cache 1 \
                        -selectioncommand [list tblCmd project %s]

pack $frame1.projTable
}

# Procedure to run after the OK button on the Login dialog is pressed
proc GoToProjectWindow {args} {
    global WEB_APP
    global COOKIES
    global PROJECTS

    # Load the value of the saved base URL
    #persistentArray settings "config.txt"
	persistentArray settings "c:/simm/config.txt"
    if {![string equal $settings(baseUrl) $::url]} {
        set settings(baseUrl) $::url
        #persistSave "config.txt" settings
		persistentArray settings "c:/simm/config.txt"
    }
    # Local proxy configuration
    #http::config -proxyhost 192.0.2.150 -proxyport 3128

    # get the full url for Login
    set loginUrl "$::url/$WEB_APP/login?user=$::user&passwd=$::pass"

    # Send the query to the server
    set token [http::geturl $loginUrl -timeout 30000]
    upvar \#0 $token state
    # Get the status of server comuunication
    set status [http::status $token]
    # Server response
    set answer [http::data $token]
    # puts $answer
    # Initiate a list for the session cookies 
    set cookies [list]
    foreach {name value} $state(meta) {
        if { $name eq "Set-Cookie" } {
            lappend COOKIES [lindex [split $value {;}] 0]
           }
    }
    ::http::cleanup $token


    if {[string equal -nocase $status ok]} {

        # Parse the returned XML file
        set loginXml [::dom::tcl::parse $answer]
        # Get the login node
        set loginNode [::dom::tcl::selectNode $loginXml "login"]
        # Get the attributes of the login node
        set loginAttr [::dom::node cget $loginNode -attributes]
        # Get the login result
        set loginResult [lindex [array get $loginAttr success] end]

        # if the login was successful send the project query
        if $loginResult {

            # Put together the project query
            set projectUrl "$::url/$WEB_APP/query?type=Project&expr=\[name==\'/Data/*\'\]&f=name&f=description"
            # send query with cookies
            set token2 [http::geturl $projectUrl -timeout 30000 -strict false -headers [list Cookie [join $COOKIES {;}]]]
            set status2 [http::status $token2]
            set answer2 [http::data $token2]
            #puts $answer2
            http::cleanup $token2

            array unset PROJECTS
            array set PROJECTS {}

            # Parse the returned XML file
            set projectXml [::dom::tcl::parse $answer2]
            # Get the object list node
            set objNode [::dom::tcl::selectNode $projectXml "objList"]
            # Get the object list children nodes
            set objListChildren [::dom::node children $objNode]
            foreach child $objListChildren {
                set tempName [::dom::node cget $child -nodeName]
                if {[string equal -nocase $tempName obj]} {
                    set tempAttr [::dom::node cget $child -attributes]
                    set tempObjId [lindex [array get $tempAttr oid] end]
                    set objListChildren2 [::dom::node children $child]
                    set tempSpec ""
                    foreach grandchild $objListChildren2 {
                        set tempName2 [::dom::node cget $grandchild -nodeName]
                        if {[string equal -nocase $tempName2 "attr"]} {
                            lappend tempSpec [::dom::node stringValue $grandchild],
                        }
                    }
                    set PROJECTS($tempObjId) $tempSpec
                }
            }
        }

    } else {
        tk_messageBox -message "The program was unable to login. Please check the user name and password and try again."
    }
    destroy $args;
    chooseProject
}


########### Initiating the Login window #########

# Define the top level widow
hwt::CreateWindow mytoplevel \
 -windowtitle "Login" \
 -noGeometrySaving \
 -cancelButton Cancel \
 -acceptButton OK \
 -acceptFunc GoToProjectWindow \
 -defaultButton Accept \
 -fixedSize 320 180 \
 -post;

# Load the value of the saved base URL
#persistentArray settings "config.txt"
persistentArray settings "c:/simm/config.txt"


# Define the recessed window area
set recess [hwt::WindowRecess mytoplevel];

# Insert section Label
set section1 [label $recess.section1 \
 -text "Login Data:" \
 -justify left \
 -anchor nw \
 -font [hwt::AppFont]];

# Pack the label
pack $section1 -side top -anchor nw;

# Add padding before the next control
hwt::AddPadding $recess -side top -height [hwt::DluHeight 4];

# Define the frame to hold the widgets
set frame1 [frame $recess.frame1];

# pack the frame
pack $frame1 -side top -anchor nw;

# Add indent spacing to the frame
hwt::AddPadding $frame1 -side left -width [hwt::DluWidth 4];

# Define the first text field for user name
set userName [ hwt::AddEntry $frame1.entry1 \
 -label "User Name:" \
 -labelWidth 15 \
 -entryWidth 42 \
 -textvariable user \
 -withoutPacking \
 -state "normal" ];

# Pack the user name field
pack $userName


# Define the second text field for password
set passwordField [ hwt::AddEntry $frame1.entry2 \
 -label "Password:" \
 -labelWidth 15 \
 -asPassword \
 -entryWidth 42 \
 -withoutPacking \
 -textvariable pass \
 -state "normal" ];

# Pack the password field
hwt::AddPadding $frame1 -side top -width [hwt::DluWidth 4];
pack $passwordField -side top -anchor nw;

# Define the second text field for password
set baseUrl [ hwt::AddEntry $frame1.entry3 \
 -label "URL:" \
 -labelWidth 15 \
 -entryWidth 42 \
 -withoutPacking \
 -textvariable url \
 -text $settings(baseUrl) \
 -state "normal" ];

# Pack the password field
hwt::AddPadding $frame1 -side top -width [hwt::DluWidth 4];
pack $baseUrl -side top -anchor nw;